#!/usr/bin/env groovy
import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.scm.*

final def ahptool = new AHPTool()
final def props = ahptool.getStepProperties()
final def solutionFile    = props['solutionFile']
final def devenvPath      = props['devenvPath']
final def command         = props['command']
final def solutionConfig  = props['solutionConfig']
final def additionalArgs  = props['additionalArgs']

final def workDir = new File('.').canonicalFile
if (workDir.isFile()) {
    throw new IllegalArgumentException("Working directory ${workDir} is a file!")
}

final def logFile = new File(workDir, "vs-plugin-out.log")

if (logFile.exists()) {
    logFile.delete()
}

// create command line
def exePath = new StringBuffer()
if (devenvPath) {
    if (devenvPath =~ "devenv([.](exe|bat|com))?\$") {
        exePath.append(devenvPath)
    }
    else {
        exePath.append(devenvPath)
            .append(File.separatorChar)
            .append("devenv")
    }
}
else {
    exePath.append("devenv")
}

def cmdArgs = [exePath, solutionFile, command, solutionConfig, "/Out", logFile.name]
if (additionalArgs) {
    cmdArgs.addAll(additionalArgs.split("\n"))
}
println "Executing:"
println "  " + cmdArgs.join(' ')
println ""

def proc = cmdArgs.execute()

proc.waitForProcessOutput(System.out, System.out)
proc.waitFor()

println "Output:"
println logFile.text
println ""

System.exit proc.exitValue()
